package com.isu.gaswellwatch.modbus.data.impl;


import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.stereotype.Component;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2025/2/22 01:07
 */
@Component(PersistenceHandler.MI_WEPS_PLUG_MODBUS_TYPE)
public class MiWepsPlugPersistenceHandler extends AbstractPersistenceHandler {

    private static final Map<String, String> MI_WEPS_PLUG_COLUMN_MAPPING_MAP = new HashMap<>();

    static {
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("report_status", "reportStatus"); // 报表状态 0:无报表数据; 1:有分钟记录; 2:有小时记录; 3:有日志记录; 4:有生产记录
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("current_status", "currentStatus"); // 当前状态 0:关井; 1:上升; 2:续流; 3:停机
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("current_time", "currentTime"); // 当前时间 "控制器时间。示例：0x4830:0x0211:0x0924:0x2013。结果：2013 - 09 - 24 11:48:30 "
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("remaining_time", "remainingTime"); // 剩余时间 "组合后时间格式：天.小时:分钟:秒。示例：0x0930:0x0001。结果：0.1:09:30 "
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("current_status_0x", "currentStatus0x"); // 当前状态0x 0x8000:关井；0x8001:上升；0x8002:续流；0x8003:停机
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("oil_pressure", "oilPressure"); // 油压 单位：MPa
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("oil_pressure_temperature", "oilPressureTemperature"); // 油压表测量温度 单位：℃
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("cas_pressure", "casPressure"); // 套压 单位：MPa
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("cas_pressure_temperature", "casPressureTemperature"); // 套压测量温度 单位：℃
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("quantity_of_electricity", "quantityOfElectricity"); // 控制器电量 单位：%
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("pluger_rising_speed", "plugerRisingSpeed"); // 柱塞上升速度 单位：米/分钟
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("gas_well_number", "gasWellNumber"); // 气井编号
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("depth_of_lock_device", "depthOfLockDevice"); // 卡定器深度 单位：米
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("arriving_sensor", "arrivingSensor"); // 到达传感器 0/1：禁用/启用
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("zigbee", "zigbee"); // Zigbee 0/1：禁用/启用
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("pulse_width", "pulseWidth"); // 脉冲宽度 驱动电磁阀使用
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("plug_rising_speed", "plugRisingSpeed"); // 上升速度 米/分钟
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("plug_too_fast_speed", "plugTooFastSpeed"); // 过快速度 米/分钟
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("plug_too_slow_speed", "plugTooSlowSpeed"); // 过缓速度 米/分钟
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("gas_well_name", "gasWellName"); // 气井名称
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("oil_and_cas_data_collection_interval", "oilAndCasDataCollectionInterval"); // 油、套压数据采集间隔 0：每5秒采样；1：每10秒采样；2：每15秒采样；3：每30秒采样；4：每1秒采样；5：每2秒采样；6：每3秒采样；7：每4秒采样
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("store_data_interval_on_open_well", "storeDataIntervalOnOpenWell"); // 数据存储间隔（开井期间） 0：每1分钟存储；1：每5分钟存储；2：每10分钟存储；3：每15分钟存储；4：每30秒存储；5：每15秒存储；6：每5秒存储
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("store_data_interval_on_close_well", "storeDataIntervalOnCloseWell"); // 数据存储间隔（关井期间） 0：每1分钟存储；1：每5分钟存储；2：每10分钟存储；3：每15分钟存储；4：每30秒存储；5：每15秒存储；6：每5秒存储
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("communication_power_threshold", "communicationPowerThreshold"); // 通信电量门限 Zigbee通讯使用
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("reserve1", "reserve1"); // 保留 Zigbee通讯使用
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("start_time_of_daytime", "startTimeOfDaytime"); // 白天起始时间 Zigbee通讯使用
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("end_time_of_daytime", "endTimeOfDaytime"); // 白天结束时间 Zigbee通讯使用
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("control_work_time", "controlWorkTime"); // 控制器工作状态 0：生产模式；1：常开模式；2：常关模式
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("produce_mode", "produceMode"); // 生产制度设置 0：定时开关井；1：时间优化模式；2：压力微升模式；3：压力回升模式；4：压力跌落模式
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("plunger_rise_time", "plungerRiseTime"); // 上升时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("dangerous_rise_time", "dangerousRiseTime"); // 危险上升时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("too_fast_rise_time", "tooFastRiseTime"); // 过快上升时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("too_slow_rise_time", "tooSlowRiseTime"); // 过缓上升时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("too_fast_count", "tooFastCount"); // 过快次数
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("too_fast_raise_open_well_time", "tooFastRaiseOpenWellTime"); // 过快增加开井时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("too_fast_lower_close_well_time", "tooFastLowerCloseWellTime"); // 过快减少关井时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("too_slow_lower_open_well_time", "tooSlowLowerOpenWellTime"); // 过缓减少开井时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("too_slow_raise_close_well_time", "tooSlowRaiseCloseWellTime"); // 过缓增加关井时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("check_stability_time", "checkStabilityTime"); // 检测稳定时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("min_close_well_time", "minCloseWellTime"); // 最小关井时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("max_close_well_time", "maxCloseWellTime"); // 最大关井时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("well_close_time_not_reached_duration", "wellCloseTimeNotReachedDuration"); // 未到达关井时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("min_open_well_time", "minOpenWellTime"); // 最小开井时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("max_open_well_time", "maxOpenWellTime"); // 最大开井时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("after_flow_time", "afterFlowTime"); // 续流时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("not_reached_flow_time", "notReachedFlowTime"); // 未到续流时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("close_time", "closeTime"); // 关井时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("open_time", "openTime"); // 开井时间 单位：秒
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("open_pressure", "openPressure"); // 开井压力 单位：MPa
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("close_pressure", "closePressure"); // 关井压力 单位：MPa
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("little_rise_pressure", "littleRisePressure"); // 微升压力 单位：MPa
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("gas_collection_mode", "gasCollectionMode"); // 集气模式 0：低压集气模式；1：高压集气模式
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("reserve2", "reserve2"); // 保留
        MI_WEPS_PLUG_COLUMN_MAPPING_MAP.put("protection_pressure", "protectionPressure"); // 保护压力 单位：MPa
    }

    @Override
    public void createTable(String tableName, Long deviceId) {
        this.createTable("sql/CREATE_MI_WEPS_PLUG.sql", tableName, deviceId);
    }

    @Override
    public Map<String, Map<String, Object>> insert(String tableName, String cacheKey) {
        String insertTableSQL = this.getResource("sql/INSERT_MI_WEPS_PLUG.sql");
        insertTableSQL = StringUtils.replace(insertTableSQL, "$TableName$", tableName);

        Map<String, Object> oldRow = this.getLastRow(tableName);
        Map<String, Object> newRow = this.redisTemplate.opsForHash().entries(cacheKey);

        this.jdbcTemplate.execute(insertTableSQL, new PreparedStatementCallback<Object>() {
            @Override
            public Object doInPreparedStatement(PreparedStatement ps) throws SQLException {
                ps.setLong(1, MiWepsPlugPersistenceHandler.this.snowflakeConfig.snowflakeId());
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 2, "deviceId", Types.BIGINT);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 3, "collectionTime", Types.TIMESTAMP);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 4, "receiveTime", Types.TIMESTAMP);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 5, "reportStatus", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 6, "currentStatus", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 7, "currentTime", Types.TIMESTAMP);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 8, "remainingTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 9, "currentStatus0x", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 10, "oilPressure", Types.DECIMAL);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 11, "oilPressureTemperature", Types.DECIMAL);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 12, "casPressure", Types.DECIMAL);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 13, "casPressureTemperature", Types.DECIMAL);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 14, "quantityOfElectricity", Types.DECIMAL);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 15, "plugerRisingSpeed", Types.DECIMAL);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 16, "gasWellNumber", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 17, "depthOfLockDevice", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 18, "arrivingSensor", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 19, "zigbee", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 20, "pulseWidth", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 21, "plugRisingSpeed", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 22, "plugTooFastSpeed", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 23, "plugTooSlowSpeed", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 24, "gasWellName", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 25, "oilAndCasDataCollectionInterval", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 26, "storeDataIntervalOnOpenWell", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 27, "storeDataIntervalOnCloseWell", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 28, "communicationPowerThreshold", Types.DECIMAL);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 29, "reserve1", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 30, "startTimeOfDaytime", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 31, "endTimeOfDaytime", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 32, "controlWorkTime", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 33, "produceMode", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 34, "plungerRiseTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 35, "dangerousRiseTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 36, "tooFastRiseTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 37, "tooSlowRiseTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 38, "tooFastCount", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 39, "tooFastRaiseOpenWellTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 40, "tooFastLowerCloseWellTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 41, "tooSlowLowerOpenWellTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 42, "tooSlowRaiseCloseWellTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 43, "checkStabilityTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 44, "minCloseWellTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 45, "maxCloseWellTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 46, "wellCloseTimeNotReachedDuration", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 47, "minOpenWellTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 48, "maxOpenWellTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 49, "afterFlowTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 50, "notReachedFlowTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 51, "closeTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 52, "openTime", Types.VARCHAR);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 53, "openPressure", Types.DECIMAL);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 54, "closePressure", Types.DECIMAL);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 55, "littleRisePressure", Types.DECIMAL);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 56, "gasCollectionMode", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 57, "reserve2", Types.INTEGER);
                MiWepsPlugPersistenceHandler.this.setValue(ps, newRow, 58, "protectionPressure", Types.DECIMAL);
                return ps.executeUpdate();
            }
        });
        if (Objects.isNull(oldRow)) {
            return Map.of("new", newRow);
        }
        return Map.of("new", newRow, "old", oldRow);
    }

    @Override
    protected Map<String, String> getColumnMappingMap() {
        return MI_WEPS_PLUG_COLUMN_MAPPING_MAP;
    }

}
