package com.isu.gaswellwatch.vo.command.weps;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.utils.ByteOrderUtils;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * 威尔普斯控制指令
 *
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2025/1/10 11:00
 */
@Getter
@Setter
@SuperBuilder
public class MiWepsPlugControl extends Command implements Timing {
    @Serial
    private static final long serialVersionUID = -7540015879950965127L;

    // 1005
    private Integer depthOfLockDevice; // 卡定器深度 单位：米 -- 气井深度

    // 1036
    private Integer controlWorkTime; // 控制器工作状态 0：生产模式；1：常开模式；2：常关模式
    // 1037
    private Integer produceMode; // 生产制度设置 0：定时开关井；1：时间优化模式；2：压力微升模式；3：压力回升模式；4：压力跌落模式

    // 1039
    private String dangerousRiseTime; // 危险上升时间 单位：秒，HH:mm:ss
    // 1040
    private String tooFastRiseTime; // 过快上升时间 单位：秒，HH:mm:ss

    // 1042
    private Integer tooFastCount; // 过快次数

    // 1062 - 2
    private String closeTime; // 关井时间 单位：秒，HH:mm:ss
    // 1064 - 2
    private String openTime; // 开井时间 单位：秒，HH:mm:ss

    // 1072
    private Integer gasCollectionMode; // 集气模式 0：低压集气模式；1：高压集气模式

    // 1074 - 2
    private BigDecimal protectionPressure; // 保护压力 单位：MPa，当集气模式为低压集气模式时，该保护压力为低压保护压力，当集气模式为高压集气模式，该保护压力为高压保护压力


    public MiWepsPlugControl() {
        this.setCode("MI_WEPS.PLUG.CONTROL");
    }

    public void setDangerousRiseTime(String dangerousRiseTime) {
        this.dangerousRiseTime = this.timingValidate(dangerousRiseTime, "dangerousRiseTime");
    }

    public void setTooFastRiseTime(String tooFastRiseTime) {
        this.tooFastRiseTime = this.timingValidate(tooFastRiseTime, "tooFastRiseTime");
    }

    public void setOpenTime(String openTime) {
        this.openTime = this.timingValidate(openTime, "openTime");
    }

    public void setCloseTime(String closeTime) {
        this.closeTime = this.timingValidate(closeTime, "closeTime");
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        List<ModbusCommandDto> commandList = new ArrayList<>(7);

        StringBuilder command = new StringBuilder(250);
        //               1005
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     03ED     0001    02
        command.append("011003ED000102").append(StringUtils.leftPad(Integer.toHexString(this.depthOfLockDevice), 4, "0"));
        commandList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //               1036
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     040C     0002    04
        command.append("0110040C000204")
                .append(StringUtils.leftPad(Integer.toHexString(this.controlWorkTime), 4, "0"))
                .append(StringUtils.leftPad(Integer.toHexString(this.produceMode), 4, "0"));
        commandList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //               1039
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     040F     0002    04
        command.append("0110040F000204")
                .append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.dangerousRiseTime)), 4, "0"))
                .append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.dangerousRiseTime)), 4, "0"));
        commandList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //               1042
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0412     0001    02
        command.append("01100412000102")
                .append(StringUtils.leftPad(Integer.toHexString(this.tooFastCount), 4, "0"));
        commandList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //               1062
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0426     0004    08
        command.append("01100426000408")
                .append(ByteOrderUtils.encode(Long.toHexString(this.toSeconds(this.closeTime)), "CDAB"))
                .append(ByteOrderUtils.encode(Long.toHexString(this.toSeconds(this.openTime)), "CDAB"));
        commandList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //               1072
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0430     0001    02
        command.append("01100430000102")
                .append(StringUtils.leftPad(Integer.toHexString(this.gasCollectionMode), 4, "0"));
        commandList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //               1074
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0432     0002    04
        command.append("01100432000204")
                .append(ByteOrderUtils.encode(Long.toHexString(Float.floatToRawIntBits(this.protectionPressure.floatValue())), "CDAB"));
        commandList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        return commandList;
    }


}
